import { isFunction } from '@antv/util';
import { jsx } from '../../jsx';

var Marker = function Marker(_ref) {
  var type = _ref.type,
      color = _ref.color;

  if (type === 'square') {
    return jsx("rect", {
      style: {
        width: '12px',
        height: '12px',
        marginRight: '10px'
      },
      attrs: {
        fill: color
      }
    });
  }

  return jsx("circle", {
    style: {
      width: '12px',
      height: '12px',
      marginRight: '10px'
    },
    attrs: {
      fill: color
    }
  });
};

export default (function (props) {
  var items = props.items,
      itemWidth = props.itemWidth,
      itemFormatter = props.itemFormatter,
      style = props.style,
      _props$marker = props.marker,
      marker = _props$marker === void 0 ? 'circle' : _props$marker;

  var formatValue = function formatValue(value) {
    if (isFunction(itemFormatter)) {
      return ": ".concat(itemFormatter(value));
    }

    return ": ".concat(value);
  };

  return jsx("group", {
    style: style
  }, items.map(function (item) {
    var color = item.color,
        name = item.name,
        value = item.value,
        filtered = item.filtered;
    return jsx("group", {
      className: "legend-item",
      style: {
        width: itemWidth,
        display: 'flex',
        flexDirection: 'row',
        alignItems: 'center',
        justifyContent: 'flex-start',
        padding: ['6px', '6px', '6px', 0]
      },
      "data-item": item
    }, jsx(Marker, {
      color: filtered ? '#bfbfbf' : color,
      type: marker
    }), jsx("text", {
      attrs: {
        fill: filtered ? '#bfbfbf' : '#808080',
        text: name
      }
    }), value ? jsx("text", {
      attrs: {
        fill: '#808080',
        text: formatValue(value)
      }
    }) : null);
  }));
});