import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import _classCallCheck from "@babel/runtime/helpers/esm/classCallCheck";
import _createClass from "@babel/runtime/helpers/esm/createClass";
import { getTickMethod } from '@antv/scale';
import { getRange } from '@antv/util';
import { toTimeStamp } from '../../util'; // 判断新老values是否相等，这里只要判断前后是否相等即可

function isValuesEqual(values, newValues) {
  if (values.length !== newValues.length) {
    return false;
  }

  var lastIndex = values.length - 1;
  return values[0] === newValues[0] && values[lastIndex] === newValues[lastIndex];
} // 不同交互之间共享的上下文


var defaultRange = [0, 1];

var Context = /*#__PURE__*/function () {
  // 交互开始时，ticks个数，主要为了每次缩放后，更新ticks个数
  // lastTickCount;
  function Context(chart) {
    _classCallCheck(this, Context);

    this.canvas = null;
    this.chart = null; // 最开始的原始值

    this.values = null; // 当前显示的范围

    this.range = defaultRange;
    this.startRange = defaultRange; // 缩放最小的点数

    this.minCount = 10; // 最小的缩放比例, 默认通过minCount计算

    this.minScale = 0.01;
    this.chart = chart;
    this.canvas = chart.context.canvas;
  } // TODO:数据或者scale发生变化后做一些处理


  _createClass(Context, [{
    key: "update",
    value: function update() {} // 缩放的主轴scale

  }, {
    key: "getPinchScale",
    value: function getPinchScale() {
      var chart = this.chart; // 默认缩放x轴

      var scales = chart.getXScales();
      return scales[0];
    } // 跟随轴的scale

  }, {
    key: "getFollowScale",
    value: function getFollowScale() {
      var chart = this.chart; // 默认缩放x轴

      var scales = chart.getYScales() || [];
      return scales[0];
    }
  }, {
    key: "start",
    value: function start() {
      var range = this.range; // const scale = this.getPinchScale();

      var _range = _slicedToArray(range, 2),
          start = _range[0],
          end = _range[1]; // 记录交互起始的范围


      this.startRange = [start, end]; // 记录开始时的ticks个数
      // this.lastTickCount = scale.tickCount;
    }
  }, {
    key: "doZoom",
    value: function doZoom(leftScale, rightScale, zoom) {
      var range = this.startRange,
          minScale = this.minScale;

      var _range2 = _slicedToArray(range, 2),
          start = _range2[0],
          end = _range2[1];

      var zoomOffset = 1 - zoom;
      var rangeLen = end - start;
      var rangeOffset = rangeLen * zoomOffset;
      var leftOffset = rangeOffset * leftScale;
      var rightOffset = rangeOffset * rightScale;
      var newStart = Math.max(0, start - leftOffset);
      var newEnd = Math.min(1, end + rightOffset);
      var newRange = [newStart, newEnd]; // 如果已经到了最小比例，则不能再继续再放大

      if (newEnd - newStart < minScale) {
        return;
      }

      this.repaint(newRange);
    }
  }, {
    key: "doMove",
    value: function doMove(ratio) {
      // 不管是0， 还是其他，都不用处理
      if (!ratio) return;
      var range = this.startRange;

      var _range3 = _slicedToArray(range, 2),
          start = _range3[0],
          end = _range3[1];

      var rangeLen = end - start;
      var rangeOffset = rangeLen * ratio;
      var newStart = start - rangeOffset;
      var newEnd = end - rangeOffset; // 处理边界值

      var newRange;

      if (newStart < 0) {
        newRange = [0, rangeLen];
      } else if (newEnd > 1) {
        newRange = [1 - rangeLen, 1];
      } else {
        newRange = [newStart, newEnd];
      }

      this.repaint(newRange);
    }
  }, {
    key: "updateRange",
    value: function updateRange(range) {
      // 0， 1 的范围之间
      var _range4 = _slicedToArray(range, 2),
          start = _range4[0],
          end = _range4[1]; // start 不能小于0


      start = Math.max(0, start); // end 不能大于1

      end = Math.min(1, end); // 设置当前的范围

      this.range = [start, end];
    }
  }, {
    key: "updateFollowScale",
    value: function updateFollowScale(pinchScale, pinchValues) {
      var chart = this.chart;
      var followScale = this.getFollowScale();
      var pinchField = pinchScale.field,
          pinchScaleType = pinchScale.type;
      var followField = followScale.field; // 根据主轴的value值，找到所有从轴的value值

      var values = []; // 转成map，让查找性能更高

      var pinchValueMap = {};
      pinchValues.forEach(function (item) {
        pinchValueMap[item] = true;
      });
      var data = chart.data;
      data.forEach(function (item) {
        if (pinchScaleType === 'timeCat') {
          var value = toTimeStamp(item[pinchField]);

          if (pinchValueMap[value]) {
            values.push(item[followField]);
          }
        }
      });

      var _getRange = getRange(values),
          min = _getRange.min,
          max = _getRange.max;

      this.updateScale(followScale, {
        min: min,
        max: max,
        nice: true
      });
    }
  }, {
    key: "updateScale",
    value: function updateScale(scale, cfg) {
      if (!scale) {
        return;
      }

      var field = scale.field;
      var chart = this.chart;
      chart.setScale(field, cfg);
    }
  }, {
    key: "getDefaultValues",
    value: function getDefaultValues() {
      if (this.values) {
        return this.values;
      }

      var _this$getPinchScale = this.getPinchScale(),
          values = _this$getPinchScale.values;

      this.values = [].concat(values);
      return this.values;
    }
  }, {
    key: "getZoomedValues",
    value: function getZoomedValues(range) {
      var values = this.getDefaultValues();

      var _range5 = _slicedToArray(range, 2),
          start = _range5[0],
          end = _range5[1];

      var len = values.length;
      var valueStart = start * len;
      var valueEnd = end * len; // 从原始数据里截取需要显示的数据

      var newValues = values.slice(valueStart, valueEnd);
      return newValues;
    }
  }, {
    key: "getZoomedTicks",
    value: function getZoomedTicks(newValues) {
      var values = this.getDefaultValues();
      var scale = this.getPinchScale();
      var tickCount = scale.tickCount; // 根据当前数据的比例，和定义的tickCount计算应该需要多少个ticks

      var newTickCount = Math.round(tickCount * values.length / newValues.length);
      var catTicks = getTickMethod('cat');
      var ticks = catTicks({
        tickCount: newTickCount,
        values: values
      });
      return ticks;
    }
  }, {
    key: "repaint",
    value: function repaint(range) {
      var scale = this.getPinchScale();

      if (!scale) {
        return;
      }

      var currentValues = scale.values,
          currentTicks = scale.ticks; // 更新range(数据窗口)

      if (range) {
        this.updateRange(range);
      } // 从原始数据里截取需要显示的数据


      var newValues = this.getZoomedValues(this.range); // 根据要展示的数据数量计算出需要展示的ticks

      var newTicks = this.getZoomedTicks(newValues); // 如果新数组和当前显示的数组相同，则不更新

      if (isValuesEqual(currentValues, newValues) && isValuesEqual(currentTicks, newTicks)) {
        return;
      } // 更新主轴度量


      this.updateScale(scale, {
        values: newValues,
        ticks: newTicks
      }); // 更新副轴度量

      this.updateFollowScale(scale, newValues); // 触发Chart重新渲染

      this.render();
    }
  }, {
    key: "render",
    value: function render() {
      var chart = this.chart;
      var animate = chart.animate;
      chart.setAnimate(false);
      chart.setState({
        zoomRange: this.range
      }, function () {
        chart.setAnimate(animate);
      });
    }
  }, {
    key: "destroy",
    value: function destroy() {}
  }]);

  return Context;
}();

export default Context;