"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.diff = diff;
exports.renderComponent = renderComponent;

var _jsx = require("../jsx");

var _util = require("@antv/util");

var _equal = _interopRequireDefault(require("./equal"));

var _createComponentTree = _interopRequireDefault(require("./createComponentTree"));

var _children = _interopRequireDefault(require("../children"));

// @ts-nocheck
function renderShape(component, children, animate) {
  // @ts-ignore
  var container = component.container,
      __lastElement = component.__lastElement,
      context = component.context; // 先清空绘制内容

  container.clear(); // children 是 shape 的 jsx 结构, component.render() 返回的结构

  var shapeElement = (0, _jsx.renderJSXElement)(children, {
    context: context
  }); // @ts-ignore

  component.__lastElement = shapeElement;
  var renderElement = animate !== false ? (0, _jsx.compareRenderTree)(shapeElement, __lastElement) : shapeElement;
  if (!renderElement) return; // 生成G的节点树, 存在数组的情况是根节点有变化，之前的树删除，新的树创建

  if ((0, _util.isArray)(renderElement)) {
    for (var i = 0, len = renderElement.length; i < len; i++) {
      (0, _jsx.render)(renderElement[i], container);
    }
  } else {
    (0, _jsx.render)(renderElement, container);
  }
}

function createComponent(parent, element) {
  var type = element.type,
      props = element.props,
      ref = element.ref;
  var container = parent.container,
      context = parent.context,
      updater = parent.updater; // 这里 一定是 F2 Component 了
  // @ts-ignore

  var component = new type(props, context, updater); // 设置ref

  if (ref) {
    ref.current = component;
  } // 每个组件都新建一个独立容器


  component.container = container.addGroup();
  component.context = context;
  component.updater = updater;
  return component;
}

function renderComponent(component) {
  _children.default.map(component, function (item) {
    var lastChildren = item.children;
    var mount = (0, _util.isUndefined)(lastChildren);

    if (mount) {
      if (item.willMount) item.willMount();
    } else if (item.willUpdate) {
      item.willUpdate();
    }
  });

  _children.default.map(component, function (item) {
    var lastChildren = item.children;
    var mount = (0, _util.isUndefined)(lastChildren);
    var newChildren = item.render(); // @ts-ignore

    if (item.isF2Container) {
      newChildren = diff(item, newChildren, lastChildren);
    } else {
      renderShape(item, newChildren, true);
    }

    component.children = newChildren;

    if (mount) {
      if (component.didMount) component.didMount();
    } else if (component.didUpdate) {
      component.didUpdate();
    }
  });
}

function renderElement(parent, element) {
  var component = element.component;

  if (component) {
    var parentGroup = parent.container;
    parentGroup.add(component.container);
  } else {
    // 把新建的component 挂到element上
    element.component = createComponent(parent, element);
  }

  renderComponent(element.component);
}

function destroyElement(elements) {
  _children.default.map(elements, function (element) {
    if (!element) return;
    var component = element.component;

    if (!component) {
      return;
    }

    if (component.willUnmount) {
      component.willUnmount();
    }

    destroyElement(component.element);
    var container = component.container;
    container.remove(true);

    if (component.didUnmount) {
      component.didUnmount();
    }
  });
}

function diffElement(parent, nextElement, lastElement) {
  if (!nextElement && !lastElement) {
    return null;
  } // 新建


  if (nextElement && !lastElement) {
    _children.default.map(nextElement, function (element) {
      renderElement(parent, element);
    });

    return;
  } // 删除


  if (!nextElement && lastElement) {
    return destroyElement(lastElement);
  } // diff


  var nextType = nextElement.type,
      nextProps = nextElement.props;
  var lastType = lastElement.type,
      lastProps = lastElement.props,
      component = lastElement.component;

  if (nextType !== lastType) {
    destroyElement(lastElement);
    renderElement(parent, nextElement);
    return;
  }

  if ((0, _equal.default)(nextProps, lastProps)) {
    return;
  }

  if (component.shouldUpdate && component.shouldUpdate(nextProps) === false) {
    return;
  }

  if (component.willReceiveProps) {
    component.willReceiveProps(nextProps);
  } // 开始更新逻辑


  component.props = nextProps;
  nextElement.component = component;
  renderElement(parent, nextElement);
  return;
}

function diff(parent, nextChildren, lastChildren) {
  var newChildren = (0, _createComponentTree.default)(nextChildren);

  _children.default.compare(newChildren, lastChildren, function (next, last) {
    diffElement(parent, next, last);
  });

  return newChildren;
}